﻿SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SR3600_fn_MPHStatistics]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[SR3600_fn_MPHStatistics]
GO

/*	Test: 
	declare @start datetime, @end datetime;
	declare @limit float;
	set @start = getdate() - 1;
	set @end = getdate();
	set @limit = 30;
	
	select * from [SR3600_fn_MPHStatistics] (@start, @end, @limit);
*/

CREATE FUNCTION [dbo].[SR3600_fn_MPHStatistics]
(
	@lowerLimit datetime,
	@upperlimit	datetime,
	@speedlimit	float = 30
)
RETURNS TABLE
AS
RETURN
(
	select 
	@lowerLimit as [begin_period],
	@upperLimit as [end_period],
	@speedlimit as [speed_limit],
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
	) as [count_all],
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > @speedlimit
	) as [count_speeders],
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > @speedlimit + 10
	) as [count_10mph_speeders],
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > @speedlimit + 20
	) as [count_20mph_speeders],
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > @speedlimit
	) * 100.0
	/
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
	) as [pct_speeders],
	(
		select 
			avg([dcs_speed_mph]-@speedlimit)
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > @speedlimit
	) as [average_speed_above_limit],
	(
		select 
			avg([dcs_speed_mph]-@speedlimit)/@speedlimit*100
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > @speedlimit
	) as [average_speed_pct_above_limit],
	(
		select 
			avg([dcs_speed_mph])
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] > 0
	) as [average_speed],
	(
		select 
			max([dcs_speed_mph])
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] between @lowerLimit and @upperLimit
	) as [max_speed],
	(
		select 
			[dcs_speed_mph]
		from 
			[dbo].[SR3600Data]
		where
			[dcs_time] = (select max([dcs_time]) from [dbo].[SR3600Data])
	) as [last_speed],
	(
		select 
			max([dcs_speed_mph])
		from 
			[dbo].[SR3600Data]
	) as [all_time_max_speed],
	(
		select 
			count(*)
		from 
			[dbo].[SR3600Data]
		where 
			[dcs_time] between @lowerLimit and @upperLimit
			and [dcs_speed_mph] = -1
	) as [bad_readings]
)
GO
