﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using DrHeet.SportsRadar.ServiceContracts;

namespace DrHeet.SportsRadarDCS
{
    [ServiceBehavior(InstanceContextMode=InstanceContextMode.Single)]
    public class SubscriptionServer : ISubscriptionServer
    {
        private static IDictionary<Guid, SportsRadarSubscriber> subscribers;

        public SubscriptionServer()
        {            
            subscribers = new Dictionary<Guid, SportsRadarSubscriber>();
        }

        public void NotifySubscribers(SR3600Manager.SpeedRecord sr)
        {
            foreach(SportsRadarSubscriber s in subscribers.Values)
            {
                try
                {
                    s.Callback.SignalSpeedData(sr);
                    s.CallCount++;
                }
                catch (Exception e)
                {
                    DCS.WriteToApplicationLog(e.Message, System.Diagnostics.EventLogEntryType.Error);
                    DCS.WriteToApplicationLog("Subscriber callback error - removed from subscribers" + s.ClientDescription + ".", System.Diagnostics.EventLogEntryType.Error);
                    UnsubscribeSpeedEvent(s.ClientId);
                }
            }
        }

        
        #region ISubScriptionServer Members

        public Guid SubscribeSpeedEvent(string clientDescription)
        {
            SportsRadarSubscriber subscriber = new SportsRadarSubscriber();
            subscriber.ClientDescription = clientDescription;
            subscriber.Callback = OperationContext.Current.GetCallbackChannel<ISportRadarDCSCallback>();

            subscribers.Add(subscriber.ClientId, subscriber);
            
            DCS.WriteToApplicationLog("Registered client " + clientDescription + ".", System.Diagnostics.EventLogEntryType.Information);

            return subscriber.ClientId;
        }


        public void UnsubscribeSpeedEvent(Guid clientId)
        {
            try
            {
                DCS.WriteToApplicationLog("Unregistered client " + subscribers[clientId].ClientDescription + ".", System.Diagnostics.EventLogEntryType.Information);

                subscribers.Remove(clientId);
            }
            catch(Exception e)
            {
                System.Diagnostics.Debug.WriteLine("Unsubscribe Error " + e.Message);
            }
        }
        #endregion
    }
}
